SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveNLU] (
    @VCC Varchar(50) = NULL,
    @Name Varchar(50) = NULL,
    @Culture Varchar(50) = NULL,
    @Description Varchar(50) = NULL,
    @AppId Varchar(100) = NULL,
    @SubscriptionKey Varchar(50) = NULL,
    @IsSystemApp BIT = 0,
    @IsPublished BIT = NULL,
    @LastPublishedDate DATETIME = NULL,
    @IsTrained BIT = NULL,
    @NeedTrain BIT = NULL,
    @NeedPublish BIT = NULL,
    @LastTrainedDate DATETIME = NULL,
    @CreatedDate Datetime = NULL,
    @CreatedByUserId Varchar(50) = NULL,
    @LastModifiedDate Datetime = NULL,
    @LastModifiedByUserId Varchar(50) = NULL,
    @TrainVersion Varchar(250) = NULL,
    @TrainId Varchar(250) = NULL,
    @UseRasaApi Bit = NULL
)
AS
BEGIN
    SET NOCOUNT ON
	DECLARE @NeedReplicateNlu BIT = 0
	DECLARE @SystemAppId VARCHAR(50)
   

	-- VERIFICAMOS SI ES PARA UN NLU DE SENTIMIENTO PERSONALIZADO
	IF NOT EXISTS( SELECT 1 FROM [NLU] WHERE [Name] LIKE 'sentiment-%' AND AppId = @AppId AND VCC =@VCC)
	BEGIN
	SET @NeedReplicateNlu = 1
	SELECT @SystemAppId = AppId FROM [NLU] WHERE [VCC] = 'system' AND [Name] = @Name
	END
		
    -- SI LA VARIABLE LLEGA NULL, LA SETTEAMOS COMO 0
    IF @IsSystemApp IS NULL
        SET @IsSystemApp = 0

    UPDATE [NLU]
    SET [Culture] = @Culture,
        [Description] = @Description,
        [AppId] = @AppId,
        [IsPublished] = CASE WHEN @IsPublished IS NULL THEN [IsPublished] ELSE @IsPublished END,
        [LastPublishedDate] = CASE WHEN @LastPublishedDate IS NULL THEN [LastPublishedDate] ELSE @LastPublishedDate END,
        [IsSystemApp] = CASE WHEN @IsSystemApp IS NULL THEN [IsSystemApp] ELSE @IsSystemApp END,
        [SubscriptionKey] = CASE WHEN @SubscriptionKey IS NULL THEN [SubscriptionKey] ELSE @SubscriptionKey END,
        [IsTrained] = CASE WHEN @IsTrained IS NULL THEN [IsTrained] ELSE @IsTrained END,
        [NeedTrain] = CASE WHEN @NeedTrain IS NULL THEN [NeedTrain] ELSE @NeedTrain END,
        [NeedPublish] = CASE WHEN @NeedPublish IS NULL THEN [NeedPublish] ELSE @NeedPublish END,
        [LastTrainedDate] = CASE WHEN @LastTrainedDate IS NULL THEN [LastTrainedDate] ELSE @LastTrainedDate END,
        -- [CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
        [LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
        [LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
        [TimeStamp] = GETUTCDATE(),
        [TrainVersion] = @TrainVersion,
        [TrainId] = @TrainId
    WHERE [VCC] = @VCC
      AND [Name] = @Name

    IF @@ROWCOUNT = 0
        BEGIN
			
            INSERT INTO [NLU] (
                [VCC], [Name], [Culture],
                [Description], [AppId], [SubscriptionKey],
                [IsSystemApp], [CreatedDate], [CreatedByUserId],
                [LastModifiedDate], [LastModifiedByUserId], [TimeStamp],[CognitiveEngine]
            )
            VALUES (
                       @VCC, @Name, @Culture,
                       @Description, @AppId, @SubscriptionKey,
                       @IsSystemApp, GETUTCDATE(), @CreatedByUserId,
                       @LastModifiedDate, @LastModifiedByUserId, GETUTCDATE(),
                       IIF(@IsSystemApp = 1, IIF(@UseRasaApi =1,'RASAAPI','RASA'), 
                       IIF(@UseRasaApi=1,'RASAAPI',(SELECT [CognitiveEngine] FROM [dbo].[VCC] WHERE [VCC] = @VCC)))
                   )

			IF(@NeedReplicateNlu = 1)
			BEGIN
				--GENERAMOS INTENCIONES DEL NUEVO NLU
                UPDATE NLU 
                SET NeedTrain = 1 
                WHERE AppId = @AppId
                
				INSERT INTO [Intent]([VCC],[Name],[IntentId],[AppId],[Color],[CreatedDate],[CreatedByUserId],
							[LastModifiedDate],[LastModifiedByUserId],[TimeStamp],[Status],[StatusCode])
				SELECT @VCC,[Name],[IntentId],@AppId,[Color],GETUTCDATE(),'system',GETUTCDATE(),'system',GETUTCDATE(),[Status],[StatusCode] 
					FROM Intent 
					WHERE [AppId] = @SystemAppId
				
				INSERT INTO [Utterance]([VCC],[AppId],[IntentId],[Id],[Text],[CreatedDate],[CreatedByUserId],
							[LastModifiedDate],[LastModifiedByUserId],[Status],[StatusCode])
				SELECT @VCC,@AppId,[IntentId],NEWID(),[Text],GETUTCDATE(),'system',GETUTCDATE(),'system',[Status],[StatusCode] 
					FROM Utterance 
					WHERE [AppId] = @SystemAppId
			END
        END

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
